#!/usr/bin/env python3

# -*- encoding: utf-8; py-indent-offset: 4 -*-

# License: GNU General Public License v2

from typing import Dict
import datetime
import pytz
import time
from dateutil import tz
import re
from cmk.agent_based.v2 import (
    AgentSection,
    CheckPlugin,
    CheckResult,
    DiscoveryResult,
    Result,
    Service,
    State,
    Metric,
    get_rate,
    get_value_store,
    GetRateError,
)

non_graphable = [
    "eDirectorySystemCurrTime",
    "eDirectoryUpTime",
    "eDirectoryAgentVersion",
    "CheckPointThreadStartTime",
    "CheckPointThreadForceStartTime",
    "CheckPointThreadIsForced",
]

uptime_attributes = [
    "eDirectoryUpTime",
]

time_attributes_ignored = [
    "BackGroundProcInterval",
    "CheckPointThreadForceStartTime",
    "CheckPointThreadStartTime",
]

time_attributes = [
    "eDirectorySystemCurrTime",
    "CheckPointThreadStartTime",
]

total_counters = [
    "inBytes",
    "outBytes",
    "Total",
    "EntryCache",
    "BlockCache",
    "TotalSize",
]

def format_partition_agent(value):
    formatted = re.sub('CN=|OU=|O=|T=', '', value).replace("=", " ").replace(" ", "").split("#")
    return formatted

def convert_timestamp(value):
    """convert Zulu time to current time in local timezone"""
    utc_dt = datetime.datetime.strptime(str(value), "%Y%m%d%H%M%SZ")
    utc = pytz.utc
    utc_dt = utc_dt.replace(tzinfo=tz.UTC)
    local_tz = tz.tzlocal()
    local_dt = utc_dt.astimezone(local_tz)
    return local_dt

def parse_ldap_data(string_table):
    """parse one lines of data to dictionary"""
    parsed = {}
    for line in string_table:
        item = line[0]
        parsed.setdefault(item, {})
        for _count, data in enumerate(line[1:]):
            if item == "Agent Partition":
                key, value=format_partition_agent(data)
            else:
                key, value=data.split("=")
            parsed[item].setdefault(key, value)
    return parsed

def discover_edirectory_items(section) -> DiscoveryResult:
    '''discover one item per key'''
    for key, data in section.items():
        yield Service(item=key)

def check_edirectory_items(item: str, section) -> CheckResult:
    value_store = get_value_store()
    data = section.get(item)
    if not data:
        return
    for key, value in data.items():
        if key in time_attributes_ignored:
            '''do not iternate keys which only hold a timestamp'''
            continue
        elif key in uptime_attributes:
            '''create readable notation of uptime attributes from seconds'''
            uptime = datetime.timedelta(seconds=float(value))
            yield Result(state=State(0), summary=f"{key}: {uptime}", details=f"{key}: {uptime}")
        elif key in time_attributes:
            '''if a timestamp attribute is not ignored, convert it to local time in (human) readable format'''
            datevalue = convert_timestamp(value)
            yield Result(state=State(0), summary=f"{key}: {datevalue}", details=f"{key}: {datevalue}")
        else:
            '''create a default result if above does not apply'''
            yield Result(state=State(0), summary=f"{key}: {value}", details=f"{key}: {value}")
        if key in total_counters:
            '''create a metric which is the difference between previous check value and current check value'''
            previous_value = value_store.get(key, 0)
            value_store[key] = value
            value_difference = float(value) - float(previous_value)
            yield Metric(key, abs(value_difference), boundaries=(0, None))
        elif key not in non_graphable:
            '''Only create a metric for graphable values'''
            yield Metric(key, float(value))

agent_section_edirectory_monitor = AgentSection(
    name="edirectory_monitor",
    parse_function=parse_ldap_data,
    )

check_plugin_edirectory_monitor = CheckPlugin(
    name="edirectory_monitor",
    service_name="%s",
    discovery_function=discover_edirectory_items,
    check_function=check_edirectory_items,
)
